# Copyright (c) 2020 Arda Seremet <ardaseremet@outlook.com>

import requests

class API:
	"""Class to interact with the API of ProgettiHWSW boards."""

	def __init__(self, ip: str):
		"""Initialize the API."""
		self.ip = ip

	def request(self, method: str, path: str, **kwargs):
		"""Make raw requests to exposed API."""
		headers = {}
		headers["requestedby"] = "pypi_api"

		try:
			return requests.request(
				method, f"{self.ip}/{path}", **kwargs, headers=headers,	timeout=5,
			)
		except:
			return False

	def execute(self, code: int, **kwargs):
		"""Make requests with API codes for boards."""
		headers = {}
		headers["requestedby"] = "pypi_api"

		try:
			return requests.request(
				"get", f"{self.ip}/index.htm?execute={code}", **kwargs, headers=headers, timeout=5,
			)
		except:
			return False


# Copyright (c) 2020 Arda Seremet <ardaseremet@outlook.com>

from ProgettiHWSW.api import API
from ProgettiHWSW.relay import Relay
from ProgettiHWSW.input import Input
from .const import STATUS_XML_PATH
from lxml import etree

class ProgettiHWSWAPI:
	"""Class to communicate with ProgettiHWSW boards."""

	def __init__(self, ip: str, is_old_board: str = "new"):
		"""Initialize the API and return the corresponding object class."""
		self.api = API(f"http://{ip}")
		self.ip = ip
		self.is_old_board = is_old_board

	def check_board(self):
		"""Check if this board is existing."""
		request = self.api.request("get", STATUS_XML_PATH)
		if request == False:
			return False

		root = etree.XML(request.text)

		is_old_board = len(root.xpath("//led0")) > 0
		relay_tags = root.xpath("//*[starts-with(local-name(), 'led')]")
		input_tags = root.xpath("//*[starts-with(local-name(), 'btn')]")
		self.is_old_board = is_old_board

		if(len(relay_tags) <= 0 and len(input_tags) <= 0):
			return False

		return {
			"title": f"{len(relay_tags)}R & {len(input_tags)}IN Board",
			"is_old": is_old_board,
			"relay_count": len(relay_tags),
			"input_count": len(input_tags)
		}

	def get_relay(self, relay_number: int, relay_mode: str = "bistable") -> Relay:
		"""Return the Relay class."""
		return Relay(self.api, relay_number, relay_mode, self.is_old_board)

	def get_input(self, input_number: int) -> Input:
		"""Return the Input class."""
		return Input(self.api, input_number, self.is_old_board)


# Copyright (c) 2020 Arda Seremet <ardaseremet@outlook.com>

from ProgettiHWSW.api import API
from ProgettiHWSW.const import STATUS_XML_PATH
import xml.etree.ElementTree as et

class Input:
	"""Clas that represents an input object."""

	def __init__(self, api: API, input_number: int, is_old: bool):
		"""Initialize Input class."""
		self.input_number = input_number
		self.api = api
		self.state = None
		self.is_old = is_old

	@property
	def id(self) -> int:
		"""Return the input number."""
		return self.input_number

	@property
	def is_on(self) -> bool:
		"""Return if the input is on."""
		return self.state

	def update(self):
		"""Update the input status."""
		request = self.api.request("get", STATUS_XML_PATH)
		resp = request.text
		root = et.fromstring(resp)
		number = self.input_number - 1 if self.is_old else self.input_number
		self.state = True if root.find(f"btn{number}").text == "up" else False


# Copyright (c) 2020 Arda Seremet <ardaseremet@outlook.com>

from ProgettiHWSW.api import API
from ProgettiHWSW.const import (TURN_ON_BASE, TURN_OFF_BASE, TEMP_MONOSTABLE_BASE, TOGGLE_BASE, STATUS_XML_PATH)
import xml.etree.ElementTree as et

class Relay:
	"""Clas that represents a relay object."""

	def __init__(self, api: API, relay_number: int, relay_mode: str, is_old: bool):
		"""Initialize Relay class."""
		self.relay_number = relay_number
		self.relay_mode = relay_mode
		self.api = api
		self.state = None
		self.is_old = is_old

	@property
	def id(self) -> int:
		"""Return the relay number."""
		return self.relay_number

	@property
	def is_on(self) -> bool:
		"""Return if the relay is on."""
		return self.state

	def toggle(self):
		"""Toggle the relay."""
		command = TOGGLE_BASE + self.relay_number 
		self.api.execute(command)

	def control(self, state: bool):
		"""Control the relay state."""
		command = ((TURN_ON_BASE if self.relay_mode == "bistable" else TEMP_MONOSTABLE_BASE) if state == True else TURN_OFF_BASE) + self.relay_number
		self.api.execute(command)

	def update(self):
		"""Update the relay status."""
		request = self.api.request("get", STATUS_XML_PATH)
		resp = request.text
		root = et.fromstring(resp)
		number = self.relay_number - 1 if self.is_old else self.relay_number
		self.state = True if root.find(f"led{number}").text == "1" else False


import os
import sys
import setuptools

def read_file(file):
	with open(os.path.join(os.path.dirname(__file__), file)) as readable:
		return readable.read()

README = read_file("README.md")

setuptools.setup(
	name='ProgettiHWSW',
	version='0.0.8',
	long_description='\n\n'.join([README]),
	long_description_content_type='text/markdown',
	description='Controls ProgettiHWSW relay boards.',
	url='http://github.com/ardaseremet/progettihwsw',
	download_url='http://github.com/ardaseremet/progettihwsw/tarball/0.0.8',
	author='Arda Seremet',
	author_email='ardaseremet@outlook.com',
	license='MIT',
	packages=setuptools.find_packages(),
	install_requires=[
		'requests',
		'lxml'
	],
	zip_safe=False,
	classifiers=[
		"License :: OSI Approved :: MIT License",
		"Development Status :: 5 - Production/Stable",
		"Programming Language :: Python :: 3.7",
		"Programming Language :: Python :: 3.8",
		"Intended Audience :: Telecommunications Industry",
		"Intended Audience :: Information Technology",
		"Intended Audience :: End Users/Desktop",
		"Intended Audience :: Legal Industry"
	]
)
